# Stubs for flask.templating (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .globals import _app_ctx_stack, _request_ctx_stack
from .signals import before_render_template, template_rendered
from jinja2 import BaseLoader, Environment as BaseEnvironment
from typing import Any

class Environment(BaseEnvironment):
    app: Any = ...
    def __init__(self, app: Any, **options: Any) -> None: ...

class DispatchingJinjaLoader(BaseLoader):
    app: Any = ...
    def __init__(self, app: Any) -> None: ...
    def get_source(self, environment: Any, template: Any): ...
    def list_templates(self): ...

def render_template(template_name_or_list: Any, **context: Any): ...
def render_template_string(source: Any, **context: Any): ...
